precision mediump float;

uniform vec2 u_resolution;
uniform float u_time;
uniform float u_progress;
uniform float u_crossfade;
uniform float u_pixelRatio;
uniform float u_frequencyHistoryIndex;
uniform sampler2D u_palette;
uniform sampler2D u_frequencies;
uniform sampler2D u_loudness;
uniform sampler2D u_frequencyHistory;

#define HALF_PI 1.5707963267948966
#define PI 3.14159265358
#define TWO_PI (2. * 3.141592653589793)

float cubicOut(float t) {
  float f = t - 1.0;
  return f * f * f + 1.0;
}

float vectorToFloat(vec4 v) {
  // See https://aras-p.info/blog/2009/07/30/encoding-floats-to-rgba-the-final
  return dot(v, vec4(1.0, 1./255.0, 1./65025.0, 1./16581375.0));
}

#define FREQ_AMPLITUDE 0
#define FREQ_PEAKS 1

float getFrequencyValue(float val, int index) {
  return vectorToFloat(texture2D(u_frequencies, vec2(val, float(index))).rgba);
}

float getTweenedLoudness(float value) {
  if (u_crossfade == 1.0) {
    // Read loudness.
    return vectorToFloat(texture2D(u_loudness, vec2(value, 0)));
  } else {
    // Mix loudness with tweening.
    return mix(vectorToFloat(texture2D(u_loudness, vec2(value, 1))), vectorToFloat(texture2D(u_loudness, vec2(value, 0))), cubicOut(u_crossfade));
  }
}

vec4 getTweenedTexture(sampler2D tex, float value) {
  if (u_crossfade == 1.0) {
    // Just read the color.
    return texture2D(tex, vec2(value, 0));
  } else {
    // Mix the color with tweening.
    return mix(texture2D(tex, vec2(value, 1)), texture2D(tex, vec2(value, 0)), vec4(cubicOut(u_crossfade)));
  }
}

// Catmull-Rom cubic spline.
vec4 cubic(float x)
{
  float s = 0.5; // potentially adjustable parameter
  float x2 = x * x;
  float x3 = x2 * x;
  vec4 w;
  w.x =    -s*x3  +      2.*s*x2 - s*x + 0.;
  w.y = (2.-s)*x3 +    (s-3.)*x2       + 1.;
  w.z = (s-2.)*x3 + (3.-2.*s)*x2 + s*x + 0.;
  w.w =     s*x3  -         s*x2       + 0.;
  return w;
}

// improved bilinear interpolated texture fetch
float getFrequencyValueBilinear(float val, int index) // (sampler2D sam, vec2 uv)
{
  vec2 res = vec2(64., 2.);
  vec2 st = vec2(val, float(index))*res - 0.5;

  vec2 iuv = floor(st);
  vec2 fuv = fract(st);

  float a = getFrequencyValue(((iuv + vec2(0.5, 0.5)) / res).x, index);
  float b = getFrequencyValue(((iuv + vec2(1.5, 0.5)) / res).x, index);

  return mix(a, b, fuv.x);
}