#pragma glslify: import('../common.glsl')

#define RESOLUTION 32.

float drawCircle(vec2 uv, vec2 point, float volume) {
    return smoothstep(volume+0.005, volume-0.005, length(point - uv));
}

void main() {
    vec2 uv = gl_FragCoord.xy/u_resolution.xy;
    uv.y /= u_resolution.x / u_resolution.y;

    vec4 color = vec4(0.);
    for (float x = 0.; x < RESOLUTION; x += 1.) {
        for (float y = 0.; y < RESOLUTION; y += 1.) {
            float halfStep = 1./RESOLUTION/2.;
            vec2 point = vec2(x/RESOLUTION, y/RESOLUTION) + vec2(halfStep);

            // Should we render this point?
            if (length(point - uv) < 0.025) {
                float historyOffset = mod(u_frequencyHistoryIndex + 1. + x, RESOLUTION)/RESOLUTION;
                float frequencyOffset = y/RESOLUTION;
                float vol = texture2D(u_frequencyHistory, vec2(frequencyOffset, historyOffset)).x;
                vec4 hue = vec4(getTweenedTexture(u_palette, vol * 1.8).rgb, 0.1);
                color += drawCircle(uv, point, vol / 20.) * hue;
            }
        }
    }

    gl_FragColor = color;
}
