#pragma glslify: import('../common.glsl')

float getTextureWithWrap(float val) {
  if (val > 0.9) {
    return mix(getFrequencyValue(val, FREQ_AMPLITUDE), getFrequencyValue(1. - val, FREQ_AMPLITUDE), (val - 0.9) * 5.);
  } else if (val < 0.1) {
    return mix(getFrequencyValue(val, FREQ_AMPLITUDE), getFrequencyValue(1. - val, FREQ_AMPLITUDE), (0.1 - val) * 5.);
  }

  return getFrequencyValue(val, FREQ_AMPLITUDE);
}

vec4 spoke(vec2 line, float angle)
{
  float len = 0.01 + getTextureWithWrap(1. - angle) * 1.4;
  float lineLength = length(line);

  vec4  color = vec4(0);
  color += smoothstep(len + 0.005, len - 0.005, lineLength);
  color *= getTweenedTexture(u_palette, len * 2.7);

  float spread = 0.03;
  color += (smoothstep(len - spread, len, lineLength - 0.01) - smoothstep(len, len+ spread, lineLength - 0.01)) * getTweenedTexture(u_palette, .2) * 0.1;

  // Clamp alpha.
  if (color.a < 0.1) {
    color.a = 0.0;
  }

  return color;
}

void main()
{
  vec2 uv = gl_FragCoord.xy / u_resolution.xy;
  uv.x *= u_resolution.x/u_resolution.y;
  vec2 line = vec2(0.5*u_resolution.x/u_resolution.y, 0.5) - uv;
  float angle = mod(atan(line.y, line.x) / TWO_PI + 0.25, 1.);

  // Mix two spokes slightly offset for antialiasing.
  gl_FragColor = mix(spoke(line, angle), spoke(line, angle + 0.0005), 0.);
  gl_FragColor.a = sqrt(length(gl_FragColor.rgb));
  // Now give everything a really vignetted overlay.
  //gl_FragColor += sqrt(uv.x*uv.y*(u_resolution.x/u_resolution.y-uv.x)*(1.-uv.y));
}
