#pragma glslify: import('../common.glsl')

#define BASE_BARS 25.0
#define SCALE 1.7
#define COLOR_SCALE (1.8/3.0)
#define COLOR_OFFSET 0.3
#define PEAK_HEIGHT 0.005

float spectrumBar(float center, float point, float width, float delta) {
  float dist = abs(point - center);
  return 1. - smoothstep(width-delta, width+delta, dist);
}

void main()
{
  float delta = 0.003 * 250.0 / u_resolution.x;

  // Adjust bars and spacing for resolution.
  float NUM_BARS = BASE_BARS;

  // Adjust for landscape.
  NUM_BARS = floor(NUM_BARS * u_resolution.x / u_resolution.y);

  vec2 uv = gl_FragCoord.xy / u_resolution.xy;
  float bar = floor(uv.x * NUM_BARS);
  float offset = bar/NUM_BARS;
  float height = getFrequencyValue(offset, FREQ_AMPLITUDE) * SCALE;
  float peak = max(getFrequencyValue(offset, FREQ_PEAKS) * SCALE, height);

  // The background.
  gl_FragColor = vec4(0.);

  // Shade the peaks.
  if (peak > 0.01) {
    gl_FragColor = 0.5 * vec4(smoothstep(peak-PEAK_HEIGHT, peak, uv.y) - smoothstep(peak, peak+PEAK_HEIGHT, uv.y));
  }

  // The bar.
  if (uv.y < height) {
    float bar = spectrumBar(offset, uv.x, 1.0/NUM_BARS - delta, delta);
    gl_FragColor = vec4(getTweenedTexture(u_palette, uv.y * COLOR_SCALE + COLOR_OFFSET).rgb, 1) * bar;
  }
}
