#pragma glslify: import('../common.glsl')

#define LINE_COUNT 20
#define THICKNESS 0.03

void main() {
  vec2 uv = gl_FragCoord.xy / u_resolution.xy;
  vec3 color = vec3(0.0);

  // Make sure treble gets the brighter colors.
  float colorMultiplier = 1.0;
  float colorOffset = 0.0;
  vec3 minColor = getTweenedTexture(u_palette, 0.0).rgb;
  vec3 maxColor = getTweenedTexture(u_palette, 1.0).rgb;

  if (length(minColor) > length(maxColor)) {
    colorMultiplier = -1.0;
    colorOffset = 1.0;
  }

  for (int index = 0; index < LINE_COUNT; index++) {
    float i = float(index);

    // Figure out amplitude of segment.
    float amplitude = getFrequencyValue(i/float(LINE_COUNT), FREQ_AMPLITUDE);
    float thickness = clamp(float(THICKNESS) - i/float(LINE_COUNT)/30.0, 0.025, 1.0);

    // Move the wave up based on amplitude.
    float waveSpeedMultiplier = (1.0 + 2.0 * i/float(LINE_COUNT));
    if (index < 5) {
      waveSpeedMultiplier = 2.0;
    } else if (index >= 5 && index < 15) {
      waveSpeedMultiplier = 1.5;
    } else {
      waveSpeedMultiplier = 1.1;
    }

    float value = sin(
      uv.x * 2.1 * (i + 1.)/1.1 + // Frequency of sine wave increases for each line.
      u_time * waveSpeedMultiplier // Move waves.
    ) * pow(amplitude, 0.9) + // Scale amplitude of wave by frequency band.
      0.5; // Move up to be centered.

    float glow = smoothstep(value-thickness, value, uv.y) - smoothstep(value, value+thickness, uv.y);
    color = max(color, glow * getTweenedTexture(u_palette, colorOffset + colorMultiplier * float(i)/float(LINE_COUNT)).rgb * (amplitude/10.0 + 0.98));
  }

  gl_FragColor = vec4(color, length(color));
}
