#pragma glslify: import('../common.glsl')

#define BOOST 1.5
#define BUCKETS 16.
#define ANTIALIAS 0.005
#define STEPS 32.
#define RESOLUTION 32.

void main() {
    vec2 uv = gl_FragCoord.xy/u_resolution.xy;
    uv.y /= u_resolution.x / u_resolution.y;

    // The background.
    gl_FragColor = vec4(0., 0., 0., 0.1);

    for (float i = 0.; i < STEPS; i += 1.) {
      float dim = 1.0 - (i/STEPS);
      float offset = i/STEPS * 0.7;
      float scale = 1. - (i/STEPS);

      float min = (1. - scale)/2.;
      float maxX = 1. - (1. - scale)/2.;

      if (uv.x >= min && uv.x <= maxX) {
          // Texture offset.
          float textureOffset = floor(clamp((uv.x - min) / scale, 0., 1.) * BUCKETS)/BUCKETS;
          float startX = min + floor(textureOffset * BUCKETS)/BUCKETS * scale;
          float bucketSize = scale / BUCKETS;

          // Read textures.
          float historyOffset = mod(u_frequencyHistoryIndex - i, STEPS)/STEPS;
          float vol = texture2D(u_frequencyHistory, vec2(textureOffset, historyOffset)).x;

          // Adjust.
          vol = (vol * BOOST * scale) / (u_resolution.x / u_resolution.y) + bucketSize;

          // Read color.
          vec4 hue = vec4(getTweenedTexture(u_palette, vol * 1.8).rgb, 0.9);

          // Draw the square.
          vec4 color = hue * (smoothstep(uv.y,              uv.y + ANTIALIAS, vol + offset) -
                              smoothstep(uv.y + bucketSize, uv.y + bucketSize + ANTIALIAS, vol + offset)) *
                             (smoothstep(startX,              startX + ANTIALIAS, uv.x) -
                              smoothstep(startX + bucketSize, startX + bucketSize + ANTIALIAS, uv.x));

          // Make more transparent.
          color.a *= pow(dim, 2.);
          color.rgb *= pow(dim, 2.);

          gl_FragColor = max(gl_FragColor, color);
      }
    }
}
