#pragma glslify: import('../common.glsl')

// The MIT License
// Copyright © 2014 Inigo Quilez
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
// associated documentation files (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the
// following conditions: The above copyright notice and this permission notice shall be included in all copies
// or substantial portions of the Software. THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#define BALLS 14

vec2 balls[BALLS];
vec3 colors[BALLS];

void main()
{
  vec2 uv = (-u_resolution.xy+2.0*gl_FragCoord.xy)/u_resolution.y;
  vec3 v = vec3(0);
  float delta = 3.141592653589793 * 2. / float(BALLS);

  for (int i=0; i<BALLS; i++) {
    // anim
    float angle = delta * float(i) + (u_time * 0.5 + getFrequencyValue(0.7, FREQ_AMPLITUDE));
    float vol = getFrequencyValue(float(i)/float(BALLS), FREQ_AMPLITUDE) * 4.0 + sin(u_time / 3.0) / 3.0;
	  balls[i] = vec2(sin(angle)*0.5, cos(angle)*0.5) * vol;
	  colors[i] = getTweenedTexture(u_palette, vol/2.5).rgb;
  }

  for (int i=0; i<BALLS; i++) {
    // potential
    v += (1.0 - smoothstep(0.2, 0.4, length(uv - balls[i]))) * colors[i];
  }

  vec3 col = mix(vec3(v), getTweenedTexture(u_palette, length(uv)).rgb, smoothstep(0.6, 0.91, v));

  gl_FragColor = vec4(col, sqrt(length(col)));
}
