#pragma glslify: import('../common.glsl')

#define BASE_BARS 24.0
#define SCALE 2.0
#define COLOR_SCALE 1.7
#define ALPHA 0.5

float circle(vec2 center, vec2 point, float radius) {
  vec2 dist = point - center;
  return 1. - smoothstep(radius-0.05, radius+0.05, dot(dist, dist) * 3.);
}

vec4 color(float NUM_BARS, vec2 uv, float scale) {
    vec2 intUV = floor(uv * NUM_BARS) / NUM_BARS;
    float height = getFrequencyValue(intUV.x*(u_resolution.y/u_resolution.x), FREQ_AMPLITUDE) * scale;

    if (height > max(intUV.y, 0.05)) {
      return vec4(getTweenedTexture(u_palette, intUV.y * COLOR_SCALE).rgb, 1.0);
    }

    return vec4(vec3(.1), 0.0);
}

void main()
{
  // Adjust for landscape.
  float NUM_BARS = BASE_BARS;
  NUM_BARS = floor(NUM_BARS * u_resolution.x / u_resolution.y);

  // Adjust scaling.
  float scale = SCALE * u_resolution.y / u_resolution.x;

  vec2 uv = gl_FragCoord.xy / u_resolution.xy;
  uv.y *= u_resolution.y / u_resolution.x;
  vec2 tiledUV = vec2(fract(uv.x * NUM_BARS), fract(uv.y * NUM_BARS));
  gl_FragColor = vec4(vec3(circle(vec2(.5), tiledUV, .5)), ALPHA);
  gl_FragColor *= vec4(color(NUM_BARS, uv, scale).rgb, 1.0);
}
