#pragma glslify: import('../common.glsl')

#define NUM_RINGS 10.
#define ANGULAR_WIDTH 0.15
#define ANGLE_OFFSET 0.25

float drawCircle(vec2 uv, float radius, float angle, float volAngle) {
    vec2 center = radius * vec2(cos(volAngle*2.*PI), sin(volAngle*2.*PI));
    float circleRadius = 0.011;
    return smoothstep(circleRadius+0.005, circleRadius-0.005, length(center - uv));
}

float drawRing(vec2 uv, float index, float angle) {
    float radius = 1.0/NUM_RINGS/2.7 * index + 0.1;
    float sampleRadius = 1.0/(NUM_RINGS-1.) * index;
    float dist = length(uv);

    float vol = getFrequencyValue(sampleRadius, FREQ_AMPLITUDE) * 4. / (sampleRadius / 1.3 + 1.);
    float timeOffset = 0.; // mod(u_time / (sampleRadius + 7.), 1.);
    float startAngle = max(0., vol + timeOffset - ANGULAR_WIDTH);
	float endAngle = vol + timeOffset;

    // Draw the grove foreground.
    float val = 0.;

	if (angle > startAngle && angle < endAngle || angle+1. > startAngle && angle+1. < endAngle) {
        val = smoothstep(radius-0.015, radius-0.007, dist) -
              smoothstep(radius+0.007, radius+0.015, dist);
    }

    // Add the rounded end-caps.
    if (vol > 0.) {
        val = max(val, drawCircle(uv, radius, angle, startAngle + ANGLE_OFFSET));
        val = max(val, drawCircle(uv, radius, angle, endAngle + ANGLE_OFFSET));
    }

    return val;
}

void main() {
    vec2 uv = gl_FragCoord.xy/u_resolution.xy;
    uv.x *= u_resolution.x/u_resolution.y;
    uv.y -= 0.5;
    uv.x -= 0.5 * (u_resolution.x/u_resolution.y);

    // Mirror in the X plane.
    uv.x *= -1.;

    vec3 color = vec3(0);
	float angle = mod(atan(uv.y, uv.x) / (2.*PI) - ANGLE_OFFSET, 1.);
  	for (float i = 0.; i <= NUM_RINGS; i += 1.) {
        color += drawRing(uv, i, angle) * getTweenedTexture(u_palette, i/(NUM_RINGS-1.)).rgb;
    }

    gl_FragColor = vec4(color, sqrt(length(color)));
}
