#pragma glslify: import('../common.glsl')

#define INNER_RADIUS 0.05
#define NUM_SPOKES 100.
#define ANGLE_OFFSET (1./NUM_SPOKES)
#define ANTIALIAS 0.0025
#define STROKE_WIDTH 0.0025

float drawArc(vec2 uv, float innerRadius, float outerRadius, float angle, float startAngle) {
    // Limit the arc from inside to outside.
    float dist = length(uv);
    float val = smoothstep(innerRadius - ANTIALIAS*2., innerRadius, dist) -
                smoothstep(outerRadius - ANTIALIAS*2., outerRadius, dist);

    // Limit the arc from edge to edge.
    float endAngle = startAngle + ANGLE_OFFSET + STROKE_WIDTH;
    val *= smoothstep(startAngle, startAngle + ANTIALIAS, angle) -
           smoothstep(endAngle - ANTIALIAS, endAngle, angle);

    return val;
}

void main() {
    // Center on 0, 0.
    vec2 uv = gl_FragCoord.xy/u_resolution.xy;
    uv.x *= u_resolution.x/u_resolution.y;
    uv.y -= 0.5;
    uv.x -= 0.5 * (u_resolution.x/u_resolution.y);
    uv.x *= -1.;

    // Convert angle to number between [0, 1], with 0 being at "noon".
    float angle = mod(atan(uv.y, uv.x) / (2.*PI) - 0.25, 1.);

    float progressRadius;
    float progressAngle = ceil(u_progress*NUM_SPOKES/100.);

    vec4 color = vec4(0.);
    for (float i=-0.2; i<NUM_SPOKES; i += 1.) {
        float segmentedAngle = 1./NUM_SPOKES*i;
        // Only process the segment if we're in the area.
        if (angle > segmentedAngle - 2./NUM_SPOKES && angle < segmentedAngle + 2./NUM_SPOKES) {
          float loudness = max(0.01, getTweenedLoudness(segmentedAngle));
          vec4  palette = vec4(getTweenedTexture(u_palette, loudness * 1.2).rgb, 0.8);
          float radius = INNER_RADIUS + loudness / 1.9;
    	  color += drawArc(uv, INNER_RADIUS, radius, angle, ANGLE_OFFSET*i) * palette;

  		  if (i + 0.2 == progressAngle) {
		    progressRadius = radius;
		  }
    	}
    }

    // Progress bar.
    color = color + drawArc(uv, progressRadius, 2., angle + 0.2/NUM_SPOKES, ANGLE_OFFSET*progressAngle) * vec4(0.5);

    gl_FragColor = color;
}
